import chimera, numpy as np

from StructBio.Scenographics.labels import LabelGroups
from StructBio.Scenographics.solids import Solids
from StructBio.Scenographics.parametricSurfaces import FrenetFrameSurface

arrows = Solids()
pieLabels = LabelGroups()
pieLabels.addLabelGroup("pieceLabels")

amounts_L = [22., 44., 55., 2., 4., 13., 22.]
amounts_L.sort()
sumAmts = sum(amounts_L)
percentAmts_L = [100. * x/sumAmts for x in amounts_L]
amounts_L = [2. * np.pi * x/sumAmts for x in amounts_L]

colors_L = []   # Use random colors biased towards green and blue: 
for i in range(len(amounts_L)):
    colors_L.append((np.random.rand()/3., np.random.rand()/2., np.random.rand()))

# Get start and stop theta values for each pie:
thetaIntervals_L = []
startTheta = 0.0
for val in amounts_L:
    endTheta = startTheta + val
    thetaIntervals_L.append((startTheta, endTheta))
    startTheta = endTheta

numPrimPts = 32
a = 10.0
b = 1.0
# Make half ellipse for primary cross section
incr = np.pi/64
seq = np.arange(-np.pi/2, np.pi/2 + incr, incr)
basePts_L = []
for theta in seq:
    basePts_L.append((a * np.cos(theta) - a, b * np.sin(theta)))

# Build pieces of pie:
for i in range(len(amounts_L)):
    spinePts_L = []
    rotMats_L = []
    incr = 2.0*np.pi/256
    for u in np.arange(thetaIntervals_L[i][0], thetaIntervals_L[i][1] + incr, incr, dtype = float):
        pt = 10.0 * np.array([np.cos(u), np.sin(u), 0.0])
        spinePts_L.append(pt)
        rotMats_L.append(np.array([[np.cos(u), np.sin(u), 0.],
                                   [0.0, 0.0, 1.0],
                                   [np.sin(u), -np.cos(u), 0.]]).transpose())

    # Push pie pieces away from pie center:
    middlePathPt = spinePts_L[len(spinePts_L)/2]
    spinePts_L = [p + middlePathPt/20. for p in spinePts_L]

    frenetPie = FrenetFrameSurface(basePts_L, spinePts_L, rotMats_L, [1.], [1.])
    frenetPie.displaySurface(colors_L[i], transparency = .8, closedBase = True,
                             startCap = True,  endCap = True, closedSpine = False)

    # Add arrow:
    arrows.addArrow(middlePathPt * 1.5, middlePathPt * 1.1, .05, (0., 0., 0.))

    # Add label:
    pieLabels.addLabel("pieceLabels", " %5.2f %%" % percentAmts_L[i], middlePathPt * 1.55)
    
arrows.display()
pieLabels.setLabels("pieceLabels", "black")
pieLabels.showLabels()

chimera.runCommand("set bg_color white")
chimera.runCommand("focus")




 
