import chimera, numpy as np
from StructBio.Scenographics.solids import Solids
from StructBio.Scenographics.parametricSurfaces import ExtrusionSurface

chimera.runCommand("set bg_color white")  # Black is more dramatic.

# Rotini:
numBasePts = 64

# Ellipse used for base curve:
a = 4.
b = 1.
basePts_L = []
for i in range(numBasePts):
    u  = 2. * i * np.pi/numBasePts
    basePts_L.append(np.array([a * np.cos(u), b * np.sin(u)]))

# Show a set of small spheres to represent points in base cross section:
spheres = Solids()
for pt in basePts_L:
    temp = list(pt)
    temp.append(0.0)
    spheres.addSphere(np.array(temp), .04, (1.0, 0., 0,))
spheres.display()

rotMats_L = []
rotIncr = 2.0*np.pi/64
for u in np.arange(0.0, 6.0*np.pi + rotIncr, rotIncr, dtype = float):
    rotMats_L.append(np.array([[np.cos(u), -np.sin(u)], 
                               [np.sin(u), np.cos(u)]]))

extrusionSurf = ExtrusionSurface(basePts_L, -16.0, 0.0, len(rotMats_L), rotMats_L, [1.], [1.])

#extrusionSurf.displayMesh((0., 1., 0.), closedBase = True)
extrusionSurf.displaySurface((.9, .9, .7), transparency = 1.0, closedBase = True)

chimera.runCommand("focus")
 
