import chimera, numpy as np
from StructBio.Scenographics.ellipsoid import Ellipsoid
from StructBio.Scenographics.parametricSurfaces import RevolutionSurface
from StructBio.Scenographics.solids import Solids

chimera.runCommand("set bg_color white")

length = 8
width = 4
height = 2

center = np.array([3., 4., 5.])

u = np.pi/8; v = np.pi/4; w = -np.pi/3
cu = np.cos(u); cv = np.cos(v); cw = np.cos(w)
su = np.sin(u); sv = np.sin(v); sw = np.sin(w)

axes_L = [np.array([cw*cv, -sw*cu + cw*sv*su,  sw*su + cw*sv*cu]),
          np.array([sw*cv,  cw*cu + sw*sv*su, -cw*su + sw*sv*cu]),
          np.array([-sv,    cv*su,             cv*cu])]

ellipsoidAxes = Solids("Ellipsoid Axes")
for a in axes_L:
    ellipsoidAxes.addSpindle(center - length * axes_L[0], center + length * axes_L[0], 0.1, (1., 0., 0.))
    ellipsoidAxes.addSpindle(center -  width * axes_L[1], center +  width * axes_L[1], 0.1, (0., 1., 0.))
    ellipsoidAxes.addSpindle(center - height * axes_L[2], center + height * axes_L[2], 0.1, (0., 0., 1.))
ellipsoidAxes.display()
    
numCrossSections = 32 # Value should be divisible by 4.
numBasePts = 32

eTest = Ellipsoid(length, width, height, center, axes_L, numCrossSections, numBasePts)

eTest.displaySurface((.5, .4, .8), transparency = .3)
eTest.displayMesh((0., 0., 0.))

# Put in arrows to represent axes of ellipsoid before it is rotated and moved.
# Show a set of small spheres to represent points in base curve:
solids = Solids("x,y, z axes")
solids.addArrow(np.array([0., 0., 0.]), np.array([1., 0., 0.]), .1, (1., 0., 0.))
solids.addArrow(np.array([0., 0., 0.]), np.array([0., 1., 0.]), .1, (0., 1., 0.))
solids.addArrow(np.array([0., 0., 0.]), np.array([0., 0., 1.]), .1, (0., 0., 1.))
solids.display()

chimera.runCommand("focus")



 
