import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import axes3d, Axes3D
from matplotlib import cm

#===============================================================================
def matplotlibSurface(x_A, y_A, z_A, windowTitle = "Surface Plot", plotTitle = ""):   
    plt.ion()

    fig = plt.figure()
    fig.canvas.set_window_title(windowTitle)
    ax = fig.gca(projection='3d', aspect = 1.0)

    surf = ax.plot_surface(x_A, y_A, z_A, rstride=1, cstride=1, cmap=cm.cool,
                           linewidth=0, antialiased=False)

    ax.set_title(plotTitle, fontsize = 14)

    fig.colorbar(surf, shrink=0.5, aspect=5)
    
    raw_input("Hit Enter to dismiss the plot.")
    plt.close()


#===============================================================================
# Mainline:

m = 20
n = 40

xVals_L = np.linspace(-1.0, 1.0, num = n, endpoint = True)
yVals_L = np.linspace(-1.0, 1.0, num = m, endpoint = True)

zVals_A = np.zeros((m, n), float)
# Saddle surface:
for i in range(m):
    for j in range(n):
        zVals_A[i, j] = np.sin(xVals_L[j]*yVals_L[i])
        
x_A, y_A = np.meshgrid(xVals_L, yVals_L) # Note both x_A and y_A have shape (m, n)

matplotlibSurface(x_A, y_A, zVals_A, plotTitle = "Saddle Surface")
