import chimera, tkMessageBox
from chimera import runCommand
from StructBio.Plots.surfacePlotKit import (TickLists, SurfacePlot, Axes, QuadrantPlanes, GridLines,
                                            TickLabels, plotMultipleSurfaces, fetchSPB)


#===============================================================================
# GUI imports:
import Tkinter as tk
import tkColorChooser
from chimera.baseDialog import ModelessDialog


class Surface3DplotGUI(ModelessDialog):
    buttons = ("Close")
    help = False    # Leave this out to get a Help button.
    title = "Surface 3D Plot"

    def fillInUI(self, parent):
        #-------------------------------------------------------------------------
        # GUI state:
        self.surfacePlotBundle = None
        self.ticks = None
        self.surface = None
        self.axes = None
        self.quadrantPlanes = None
        self.gridLines = None
        self.tickLabels = None
        self.tickLabelsShow_D = {"xT_yMax":  True, "yT_xMax":  True, "zT_xMax": False,
                                 "xT_zMax": False, "yT_zMax": False, "zT_yMax": True}
        self.tickLabelColors_D = {"xT_yMax": "red", "yT_xMax": "forest green", "zT_xMax": "blue",
                                  "xT_zMax": "red", "yT_zMax": "forest green", "zT_yMax": "blue"}
        self.tickLabelStride_D = {"xT_yMax": 1, "yT_xMax": 1, "zT_xMax": 1,
                                  "xT_zMax": 1, "yT_zMax": 1, "zT_yMax": 1}
        
        #-------------------------------------------------------------------------
        # FETCH FRAME
        fetchSPB_frame = tk.LabelFrame(parent, text =  " Generate Surface Plot ", bd = 2, relief = tk.RIDGE)
        fetchSPB_frame.grid(row = 0, column = 0, padx = 4, pady = 4, sticky = tk.W + tk.E)

        fetchSPB_text = tk.Label(fetchSPB_frame, text =  " Surface data:   ")
        fetchSPB_text.grid(row = 0, column = 0, pady = 4, sticky = tk.W)

        self.fetchSPB_btn = tk.Button(fetchSPB_frame, text = " Fetch ",
                                      command = self.fetchSPB_btn_Func)
        self.fetchSPB_btn.grid(row = 0, column = 1, padx = 4, pady = 4)

        #-------------------------------------------------------------------------
        # SURFACE FRAME
        surfaceColor_frame = tk.LabelFrame(parent, text =  " Surface Color Selection ", bd = 2, relief = tk.RIDGE)
        surfaceColor_frame.grid(row = 1, column = 0, padx = 4, pady = 4, sticky = tk.W + tk.E)
  
        # Initial range colors:
        self.zRangeColors_D = {"Low z" :(128, 128, 128), "High z" :(128, 128, 128)}
        self.surfaceColorButtons_D = {"Low z": None, "High z": None}

        # Radio buttons to choose spb intrinsic color or z range color:
        self.spb_zRange_Rbtn_Result = tk.StringVar()
        self.spb_Rbtn = tk.Radiobutton(surfaceColor_frame, text = "spb          ", state = tk.DISABLED, #indicatoron = 0,
                                       variable = self.spb_zRange_Rbtn_Result, value = "spb",
                                       command = self.spb_zRange_Rbtns_Func)
        self.spb_Rbtn.grid(row = 0, column = 0, padx = 4, pady = 6, sticky = tk.E)
                
        self.zRange_Rbtn = tk.Radiobutton(surfaceColor_frame, text = "z range    ", state = tk.DISABLED, #indicatoron = 0,
                                          variable = self.spb_zRange_Rbtn_Result, value = "zRange",
                                          command = self.spb_zRange_Rbtns_Func)
        self.zRange_Rbtn.grid(row = 0, column = 1, pady = 6, sticky = tk.E)
        self.spb_zRange_Rbtn_Result.set("spb") # This button starts off selected.         

        # Build two buttons on the same row:
        colPos = 2
        for endName in ("Low z", "High z"):
            tk.Label(surfaceColor_frame, text =  "  " + endName + ": ").grid(row = 0, column = colPos, pady = 4, sticky = tk.W)
            btn = tk.Button(surfaceColor_frame, text = "", width = 2, height = 1, bg = "#%02x%02x%02x" % self.zRangeColors_D[endName])
            btn.configure(command = lambda aName = endName: self.zRangeColor_btns_Func(aName))
            btn.grid(row = 0, column = colPos + 1, padx = 4, pady = 4)
            self.surfaceColorButtons_D[endName] = btn
            colPos += 2

        
        #-------------------------------------------------------------------------
        # AXES FRAME:
        axes_frame = tk.LabelFrame(parent, text =  " Axes ", bd = 2, relief = tk.RIDGE)
        axes_frame.grid(row = 2, column = 0, padx = 4, pady = 4, sticky = tk.W + tk.E)

        # User interaction label:
        tk.Label(axes_frame, text =  " Change colors:    ").grid(row = 0, column = 0, pady = 4, sticky = tk.W)

        # Initial colors for axes:
        self.axesColors_D = {"x": (255, 0, 0), "y": (0, 255, 0), "z":(0, 0, 255)}
        self.axesButtons_D = {"x": None, "y": None, "z": None}

        # Build three buttons on the same row:
        colPos = 1
        for axisName in ("x", "y", "z"):
            tk.Label(axes_frame, text =  "        " + axisName + ": ").grid(row = 0, column = colPos, pady = 4, sticky = tk.W)
            btn = tk.Button(axes_frame, text = "", width = 2, height = 1, bg = "#%02x%02x%02x" % self.axesColors_D[axisName])
            btn.configure(command = lambda aName = axisName: self.axisColor_btns_Func(aName))
            btn.grid(row = 0, column = colPos + 1, padx = 4, pady = 4)
            self.axesButtons_D[axisName] = btn
            colPos += 2


        #-------------------------------------------------------------------------
        # QUADRANT PLANES FRAME:
        quadPlanes_frame = tk.LabelFrame(parent, text =  " Quadrant Planes ", bd = 2, relief = tk.RIDGE)
        quadPlanes_frame.grid(row = 3, column = 0, padx = 4, pady = 4, sticky = tk.W + tk.E)

        # User interaction label:
        tk.Label(quadPlanes_frame, text =  " Change colors:    ").grid(row = 0, column = 0, pady = 4, sticky = tk.W)

        # Initial colors for quadrant planes:        
        self.quadPlanesColors_D = {"yz": (255, 0, 0), "zx": (0, 255, 0), "xy":(0, 0, 255)}
        self.planesButtons_D = {"yz": None, "zx": None, "xy": None}

        # Build three buttons on the same row:
        colPos = 1
        for planeName in ("yz", "zx", "xy"):
            tk.Label(quadPlanes_frame, text =  "       " + planeName + ": ").grid(row = 0, column = colPos, pady = 4, sticky = tk.W)
            btn = tk.Button(quadPlanes_frame, text = "", width = 2, height = 1, bg = "#%02x%02x%02x" % self.quadPlanesColors_D[planeName])
            btn.configure(command = lambda aName = planeName: self.quadPlaneColor_btns_Func(aName))
            btn.grid(row = 0, column = colPos + 1, padx = 3, pady = 4)
            self.planesButtons_D[planeName] = btn
            colPos += 2

        # Transparency for all planes
        self.transparency_scaleVar = tk.DoubleVar()
        transparency_text = tk.Label(quadPlanes_frame, text =  " Transparency:")
        transparency_text.grid(row = 1, column = 0, pady = 4, sticky = tk.W)
        transparency_scale = tk.Scale(quadPlanes_frame, variable = self.transparency_scaleVar,
                                      orient = tk.HORIZONTAL, length = 100, from_ = 0.0, to = 1.0, resolution = 0.1,
                                      troughcolor = "white")
        transparency_scale.grid(row = 1, column = 1, columnspan = 3, sticky = tk.W)
        transparency_scale.set(0.2)
        
        transparencyAccept_btn = tk.Button(quadPlanes_frame, text = "Accept", command = self.transparencyAccept_btn_Func)
        transparencyAccept_btn.grid(row = 1, column = 4, columnspan = 2, padx = 4, pady = 4, sticky = tk.S + tk.W)


        #-------------------------------------------------------------------------
        # NUMBER OF GRID TICKS FRAME:
        numTicks_frame = tk.LabelFrame(parent, text =  " Grid Ticks ", bd = 2, relief = tk.RIDGE)
        numTicks_frame.grid(row = 4, column = 0, padx = 4, pady = 4, sticky = tk.W + tk.E)

        # User interaction label:
        tk.Label(numTicks_frame, text =  " Number of ticks:  ").grid(row = 0, column = 0, columnspan = 3,  pady = 4, sticky = tk.W)

        # Build three spinboxes on the same row followed by an accept button:
        self.spinBoxes_D = {"x": tk.StringVar(), "y": tk.StringVar(), "z": tk.StringVar()}
        self.currentNumTicks_D = {"x": None, "y": None, "z": None}
        colPos = 0
        for spinboxLabel in ("x", "y", "z"):
            tk.Label(numTicks_frame, text = "    " + spinboxLabel + ": ").grid(row = 1, column = colPos)
            sbx = tk.Spinbox(numTicks_frame, width = 3, from_ = 3, to = 36, increment = 1, bg = "white",
                             textvariable = self.spinBoxes_D[spinboxLabel])
            sbx.grid(row = 1, column = colPos + 1)
            self.currentNumTicks_D[spinboxLabel] = "10"  # These may change when a surface plot bundle is fetched...
            self.spinBoxes_D[spinboxLabel].set(self.currentNumTicks_D[spinboxLabel])
            colPos += 2
        self.numTicksAccept_btn = tk.Button(numTicks_frame, text = "Accept", state = tk.DISABLED,
                                            command = self.numTicksAccept_btn_Func)
        self.numTicksAccept_btn.grid(row = 1, column = 7, columnspan = 2, padx = 8, pady = 4, sticky = tk.E)  


        #-------------------------------------------------------------------------
        # TICK LABELS FRAME:
        tickLabels_frame = tk.LabelFrame(parent, text =  " Grid tick labels ", bd = 2, relief = tk.RIDGE)
        tickLabels_frame.grid(row = 5, column = 0, padx = 4, pady = 4, sticky = tk.W + tk.E)
        
        gridPicture_subFrame = tk.Frame(tickLabels_frame, bd = 2, relief = tk.RIDGE, bg = "white")
        gridPicture_subFrame.grid(row = 0, column = 0, rowspan = 3, padx = 4, pady = 4, sticky = tk.W)        
        gridPicture_canvas = tk.Canvas(gridPicture_subFrame, width = 148, height = 120)
        gridPicture_canvas.grid(row = 0, column = 1, rowspan = 3, pady = 4, sticky = tk.W)
        gfImage = tk.PhotoImage(file = "gridPicture.gif")
        gridPicture_canvas.create_image(0, 0, image = gfImage, anchor = tk.NW)
        # Store a reference to the image as an attribute of the canvas:
        gridPicture_canvas.img = gfImage  # Remove this and the image disappears.

        self.selectedTicks_Rbtn_Result = tk.StringVar()
        gridRadioBtn_L = [("", "xT_zMax", 0, 0), ("", "yT_zMax", 0, 2),
                          ("", "zT_xMax", 1, 0), ("", "zT_yMax", 1, 2),
                          ("", "yT_xMax", 2, 0), ("", "xT_yMax", 2, 2)]
        

        for text, rslt, rw, cl in gridRadioBtn_L:
            rBtn = tk.Radiobutton(gridPicture_subFrame, text = text, variable = self.selectedTicks_Rbtn_Result,
                                  bg = "white", value = rslt, command = self.chooseTicks_Rbtns_Func)
            rBtn.grid(row = rw, column = cl, sticky = tk.W + tk.E)
        self.selectedTicks_Rbtn_Result.set("zT_yMax") # This button starts off selected.
        
        self.tickLabels_ShowHide_Rbtn_Result = tk.StringVar()
        self.showTickLabels_Rbtn = tk.Radiobutton(tickLabels_frame, text = "Show", state = tk.DISABLED,
                                                  variable = self.tickLabels_ShowHide_Rbtn_Result,
                                                  value = "Show", command = self.tickLabels_Rbtns_Func)
        self.showTickLabels_Rbtn.grid(row = 0, column = 1, pady = 6, sticky = tk.W)
                
        self.hideTickLabels_Rbtn = tk.Radiobutton(tickLabels_frame, text = "Hide", state = tk.DISABLED,
                                                  variable = self.tickLabels_ShowHide_Rbtn_Result,
                                                  value = "Hide", command = self.tickLabels_Rbtns_Func)
        self.hideTickLabels_Rbtn.grid(row = 0, column = 2, pady = 6, sticky = tk.W)
        self.tickLabels_ShowHide_Rbtn_Result.set("Show") # This button starts off selected.        

        # Set up stride spin box:
        tk.Label(tickLabels_frame, text =  " Set tick label stride:  ").grid(row = 1, column = 1, columnspan = 2,
                                                                           sticky = tk.W + tk.S)
        self.stride_spinVar = tk.StringVar()
        tk.Spinbox(tickLabels_frame, from_ = 1, to = 10, increment = 1, textvariable = self.stride_spinVar,
                   width = 3, bg = "white").grid(row = 2, column = 1, padx = 6, pady = 8, sticky = tk.N + tk.W)
        self.stride_spinVar.set('1')
        
        self.strideAccept_btn = tk.Button(tickLabels_frame, text = "Accept", state = tk.DISABLED,
                                          command = self.strideAccept_btn_Func)
        self.strideAccept_btn.grid(row = 2, column = 2, padx = 4, pady = 4, sticky = tk.W + tk.N)


    #=============================================================================
    # CALLBACK functions:
    #-------------------------------------------------------------------------
    def fetchSPB_btn_Func(self):
        self.surfacePlotBundle = fetchSPB()
        if self.surfacePlotBundle == None: return
        else:
            self.fetchSPB_btn.configure(state = tk.DISABLED)
        self.surfaceColorButtons_D["Low z"].configure(bg = "#%02x%02x%02x" % self.zRangeColors_D["Low z"])
        self.surfaceColorButtons_D["High z"].configure(bg = "#%02x%02x%02x" % self.zRangeColors_D["High z"])
        self.ticks = TickLists([self.surfacePlotBundle], 10) # Initial numTicks for longest side is 10.
        self.surface = plotMultipleSurfaces([self.surfacePlotBundle], self.ticks, spbColoration = True)[0]
        self.spbColoration = True
        self.spinBoxes_D["x"].set(self.ticks.num_xTicks)
        self.spinBoxes_D["y"].set(self.ticks.num_yTicks)
        self.spinBoxes_D["z"].set(self.ticks.num_zTicks)        
        self._updateAxes()
        self._updateQuadPlanes()
        self._updateGridLines()
        self._updateTickLabels()
        self.spb_Rbtn.configure(state = tk.NORMAL)
        self.zRange_Rbtn.configure(state = tk.NORMAL)
        self.numTicksAccept_btn.configure(state = tk.NORMAL)
        self.showTickLabels_Rbtn.configure(state = tk.NORMAL)
        self.hideTickLabels_Rbtn.configure(state = tk.NORMAL)
        self.strideAccept_btn.configure(state = tk.NORMAL)

    #-------------------------------------------------------------------------
    def spb_zRange_Rbtns_Func(self):
        self.spbColoration = self.spb_zRange_Rbtn_Result.get() == "spb"
        self._updateSurface()

    
    #-------------------------------------------------------------------------
    def zRangeColor_btns_Func(self, endName):
        (rgb, hx) = tkColorChooser.askcolor(initialcolor = self.zRangeColors_D[endName], title = "Choose a color")
        self.zRangeColors_D[endName] = rgb
        self.surfaceColorButtons_D[endName].configure(bg = hx)
        if self.surface and not self.spbColoration:
            self._updateSurface()


    #-------------------------------------------------------------------------
    def axisColor_btns_Func(self, axisName):
        (rgb, hx) = tkColorChooser.askcolor(initialcolor = self.axesColors_D[axisName], title = "Choose a color")
        self.axesColors_D[axisName] = rgb
        self.axesButtons_D[axisName].configure(bg = hx)
        if self.ticks and self.axes: 
            eliminate(self.axes)
            self._updateAxes()


    #-------------------------------------------------------------------------
    def quadPlaneColor_btns_Func(self, planeName):
        (rgb, hx) = tkColorChooser.askcolor(initialcolor = self.quadPlanesColors_D[planeName], title = "Choose a color")
        self.quadPlanesColors_D[planeName] = rgb
        self.planesButtons_D[planeName].configure(bg = hx)
        if self.ticks and self.axes: 
            eliminate(self.quadrantPlanes)
            self._updateQuadPlanes()


    #-------------------------------------------------------------------------
    def transparencyAccept_btn_Func(self):
        if self.ticks and self.quadrantPlanes:
            eliminate(self.quadrantPlanes)
            self._updateQuadPlanes()
            

    #-------------------------------------------------------------------------
    def numTicksAccept_btn_Func(self):
        # Current setting for numTicks per side:
        curNumTicks_L = [self.currentNumTicks_D[k] for k in ("x", "y", "z")]
        newNumTicks_L = []
        for axisLabel in ("x", "y", "z"):
            try:
                newNum = int(self.spinBoxes_D[axisLabel].get())
            except ValueError:
                tkMessageBox.showwarning(title = "Warning",
                                         message = "Entry must be an integer between 3 and 99 inclusive")
                break
            else:
                if 3 <= newNum and newNum < 100:
                    newNumTicks_L.append(newNum)
                else:
                    tkMessageBox.showwarning(title = "Warning",
                                             message = "Entry must be an integer between 3 and 99 inclusive")

        if len(newNumTicks_L) == 3:
            self.currentNumTicks_D["x"] = newNumTicks_L[0]
            self.currentNumTicks_D["y"] = newNumTicks_L[1]
            self.currentNumTicks_D["z"] = newNumTicks_L[2]
            self.ticks.num_xTicks = self.currentNumTicks_D["x"]
            self.ticks.num_yTicks = self.currentNumTicks_D["y"]                    
            self.ticks.num_zTicks = self.currentNumTicks_D["z"]
            self.ticks.updateTickLists()
            eliminate(self.axes)
            self._updateAxes()          
            eliminate(self.quadrantPlanes)
            self._updateQuadPlanes()
            eliminate(self.gridLines)
            self._updateGridLines()
            eliminateTickLabels(self.tickLabels)
            self._updateTickLabels()
            

    #-------------------------------------------------------------------------
    def chooseTicks_Rbtns_Func(self):
        # Set up show/hide radio buttons and the stride spinbox to reflect current state.
        tickName = self.selectedTicks_Rbtn_Result.get()
        if self.tickLabelsShow_D[tickName]:
            self.tickLabels_ShowHide_Rbtn_Result.set("Show")
        else:
            self.tickLabels_ShowHide_Rbtn_Result.set("Hide")

        self.stride_spinVar.set(str(self.tickLabelStride_D[tickName]))


    #-------------------------------------------------------------------------
    def tickLabels_Rbtns_Func(self):
        self.tickLabelsShow_D[self.selectedTicks_Rbtn_Result.get()] = self.tickLabels_ShowHide_Rbtn_Result.get() == "Show"
        self._updateTickLabelStride()


    #-------------------------------------------------------------------------
    def strideAccept_btn_Func(self):
        tickName = self.selectedTicks_Rbtn_Result.get()
        try:
            newStride = int(self.stride_spinVar.get())
        except ValueError:
            tkMessageBox.showwarning(title = "Warning",
                                     message = "Entry must be an integer between 1 and 10 inclusive")
            return
        else:
            if 1 <= newStride and newStride <= 10:
                self.tickLabelStride_D[tickName] = newStride
            else:
                tkMessageBox.showwarning(title = "Warning",
                                         message = "Entry must be an integer between 1 and 10 inclusive")
        self._updateTickLabelStride()        
        


    #-------------------------------------------------------------------------
    # PRIVATE functions:
    #-------------------------------------------------------------------------
                             
    def _updateSurface(self):
        if self.surface:
            eliminate(self.surface)
            if self.spbColoration:
                self.surface = plotMultipleSurfaces([self.surfacePlotBundle], self.ticks, spbColoration = True)[0]
            else:
                self.surface = plotMultipleSurfaces([self.surfacePlotBundle], self.ticks, spbColoration = False,
                                                    lowZendColor = tuple(float(c/255.) for c in self.zRangeColors_D["Low z"]),
                                                    highZendColor = tuple(float(c/255.) for c in self.zRangeColors_D["High z"]))[0]
        
    #-------------------------------------------------------------------------
    def _updateAxes(self):
        self.axes = Axes(self.ticks,
                         xAxisColor = tuple(float(c/255.) for c in self.axesColors_D["x"]),
                         yAxisColor = tuple(float(c/255.) for c in self.axesColors_D["y"]),
                         zAxisColor = tuple(float(c/255.) for c in self.axesColors_D["z"]))        
        self.axes.display()
            
    #-------------------------------------------------------------------------                             
    def _updateQuadPlanes(self):
        self.quadrantPlanes = QuadrantPlanes(self.ticks,
                                             yzPlaneColor = tuple(float(c/255.) for c in self.quadPlanesColors_D["yz"]),
                                             zxPlaneColor = tuple(float(c/255.) for c in self.quadPlanesColors_D["zx"]),
                                             xyPlaneColor = tuple(float(c/255.) for c in self.quadPlanesColors_D["xy"]),
                                             transparency = self.transparency_scaleVar.get())
        self.quadrantPlanes.display()

    #-------------------------------------------------------------------------                             
    def _updateGridLines(self):
        self.gridLines = GridLines(self.ticks)
        self.gridLines.display()

    #-------------------------------------------------------------------------                             
    def _updateTickLabels(self):
        self.tickLabels = TickLabels(self.ticks)
        self.tickLabels.setLabels("axesLabels", "black")

        self._updateTickLabelStride()

    #----------------------------------------------------------------------
    def _updateTickLabelStride(self):
        for tickName in ("xT_yMax", "yT_xMax", "zT_xMax", "xT_zMax", "yT_zMax", "zT_yMax"):
            self.tickLabels.clearLabels(tickName)
            if  self.tickLabelsShow_D[tickName]:
                self.tickLabels.setLabels(tickName, self.tickLabelColors_D[tickName],
                                          self.tickLabelStride_D[tickName])



#===============================================================================
# Global functions:

def eliminate(model):
    if model.surfFlag:
        chimera.openModels.close([model.m])
    else:
        chimera.openModels.close([model.vrmlModel])
        

def eliminateTickLabels(model):
    chimera.openModels.close([model.m])
        
#===============================================================================
# Mainline:

runCommand("set bg_color white")

spbGUI = Surface3DplotGUI()

