import chimera
from numpy import array, zeros

# For documentation purposes we use the following suffix notations:
# _A designates an Array.
# _D designates a Dictionary.
# _L designates a List.


# -----------------------------------------------------------------------------
# This function computes the centroid of atom coordinates stored as a list.
# Each 3D coordinate is assumed to be a numpy array.
def centroid(atomCoords):
    sum = zeros(3, float)
    for coord in atomCoords:
        sum += coord
    return sum/float(len(atomCoords))
    
prot = chimera.openModels.open('1FFK', type="PDB")[0]

# Get list of chain identifiers:
chainId_L = prot.sequences(True).keys()

# Initialize lists of coordinates to be empty lists:
allAtoms_L = []
alphasCs_L = []

# Build dictionaries to hold lists of coordinates for all atoms in the chains
# and alpha carbons in the chains. A dictionary key will be a chainId and the corresponding entry
# will be the list of atoms (all atoms or alpha carbons depending on the dictionary).
allAtomsInChain_D = {}
alphaCarbsInChain_D = {}

# Initiallize each dictionary entry to an empty list.
for chainId in chainId_L:
    allAtomsInChain_D[chainId] = []
    alphaCarbsInChain_D[chainId] = []
    
# Now build all the lists of coordinates:
for atom in prot.atoms:
    atomCoord_A = array(atom.coord())
    
    allAtoms_L.append(atomCoord_A)
    allAtomsInChain_D[atom.residue.id.chainId].append(atomCoord_A)
    
    if atom.name == 'CA':
        alphasCs_L.append(atomCoord_A)
        alphaCarbsInChain_D[atom.residue.id.chainId].append(atomCoord_A)

print "Centroid of all atoms is: ", centroid(allAtoms_L)
print "Centroid of all alpha carbons is: ", centroid(alphasCs_L)
for chainId in chainId_L:
    print "Centroid of all atoms in chain ", chainId, " is: ", centroid(allAtomsInChain_D[chainId])
for chainId in chainId_L:
    print "Centroid of alpha carbons in chain ", chainId, " is: ", centroid(alphaCarbsInChain_D[chainId])
