import chimera, numpy
from numpy import *

# -----------------------------------------------------------------------------
# Generate a "label" character string for a residue:
def resInfo(rr):
    return rr.type + " " + str(rr.id.position) + "." + rr.id.chainId

#==============================================================================
# Mainline:
#==============================================================================

pdbID = raw_input("Type in a PDB ID \n")
prot = chimera.openModels.open(pdbID, type = 'PDB')[0]

noBridgeResidues_L = []
bridges_D = {}

for r in prot.residues:
    if r.type != "CYS": continue
    sulfurAtom = r.findAtom('SG')

    bridgeFound = False
    for sNbr in sulfurAtom.neighbors:
        if sNbr.name == 'SG':
            bridgeFound = True
            bridgePartner = sNbr

    if not bridgeFound:
        noBridgeResidues_L.append(r)
        continue

    partnerRes = bridgePartner.residue
    # If we get this far, we have a cysteine bridge.
    # To process a bridge only once, we demand that the residue
    # holding the other SG atom has a position not already processed:
    if r.id.chainId > partnerRes.id.chainId: continue
    if r.id.position > partnerRes.id.position: continue

    # Compute distance between alpha carbon atoms in the CYS residues.
    alphaDiff = array(r.findAtom('CA').coord()) - array(partnerRes.findAtom('CA').coord())
    bridges_D[(r, partnerRes)] = numpy.linalg.norm(alphaDiff)


# Report results:
if len(bridges_D) > 0:
    print "\nDisulfide bridges: \n"
    print "      Residue pairs          Distance between"
    print "                               alpha carbons\n"
    for k in bridges_D.keys():
        resLabel = resInfo(k[0])
        parLabel = resInfo(k[1])
        print resLabel, "\t", parLabel, " \t %7.3f" % bridges_D[k]

if len(noBridgeResidues_L) > 0:
    print "\n\nThere is no disulfide bridge for each of the following: \n"
    for r in noBridgeResidues_L:
        print  resInfo(r)
