import chimera, numpy
from chimera import runCommand
from numpy import array

from StructBio.HBonds import Dehydrons
from StructBio.Utilities.hiLite import HiLiter
from StructBio.Scenographics.ellipsoid import Ellipsoid
from StructBio.Scenographics.solids import Solids

#===============================================================================
# Mainline:
#===============================================================================
sphereRadius = 7.0
#sphereRadius = 6.5
pdbIDchars = raw_input("Type in 4 character PDB ID or PDB file name: \n")
prot = chimera.openModels.open(pdbIDchars, type="PDB")[0]

dehydrons = Dehydrons(prot, sphereRadius = sphereRadius)
hiLiter = HiLiter(prot, "white")
# Give all underwrapped hydrogen bonds a green color:
for (donor, acptr) in dehydrons.dehydrons_L:
    hBondSpec = ("#" + str(prot.id) + ":" + str(donor.residue.id.position) + "@N" +
                 "#" + str(prot.id) + ":" + str(acptr.residue.id.position) + "@O")
    runCommand("setattr p color green " + hBondSpec)

print "Protein: ", pdbIDchars, "  Number of dehydrons: ", len(dehydrons.dehydrons_L)
# Go through the protein with hiLiter.  Atoms specifying the current hydrogen bond
# are represented in stick form while the carbonaceous atoms are done in ball and stick.
for (donor, acptr) in dehydrons.hBonds_L:
    print "Donor: ", donor.residue.type, str(donor.residue.id.position), donor.residue.id.chainId, \
          "     Acceptor: ", acptr.residue.type, str(acptr.residue.id.position), acptr.residue.id.chainId, "     Wrap count: ", len(dehydrons.wrapper_D[(donor, acptr)])
    hiLiter.emphasize([e for e in dehydrons.wrapper_D[(donor, acptr)]], [donor.residue.findAtom("CA"), acptr.residue.findAtom("CA")])
    donorSphere = Ellipsoid(sphereRadius, sphereRadius, sphereRadius, array(donor.residue.findAtom("CA").coord()),
                            [array((0., 0., 1.)), array((0., 1., 0.)), array((1., 0., 0.))], 32, 32, "Donor Sphere")
    acptrSphere = Ellipsoid(sphereRadius, sphereRadius, sphereRadius, array(acptr.residue.findAtom("CA").coord()),
                            [array((0., 0., 1.)), array((0., 1., 0.)), array((1., 0., 0.))], 32, 32, "Acceptor Sphere")
    donorSphere.displaySurface((.0, .0, .6), .3)
    acptrSphere.displaySurface((.6, .0, .0), .3)
    hbSpindle = Solids()
    if (donor, acptr) in dehydrons.dehydrons_L:
        spindleColor = (0.0, 1.0, 0.0)
    else:
        spindleColor = (0.5, 0.5, 0.5)
    hbSpindle.addSpindle(array(donor.labelCoord()), array(acptr.labelCoord()), 0.25, spindleColor)
    # Note: End coordinates for the spindle are not suitable for ribbon representation which would require labelCoord().
    hbSpindle.display()
    raw_input("Hit Entry key for next hydrogen bond.\n")
    chimera.openModels.close([chimera.openModels.list()[i] for i in range(1,4)])
    # Note: this last statement assumes tha user has not introduced any other models such as a molecular surface...
    
hiLiter.clearEmphasis()



    
