import chimera, numpy
from chimera import runCommand, selection
from numpy import array
import Tkinter as tk

from StructBio.ModelessDialogs.chainsFetcherBase import ChainsFetcherBase
from StructBio.HBonds import Dehydrons
from StructBio.Utilities.miscFunctions import atomIdString
from StructBio.Utilities.hiLite import HiLiter
from StructBio.Scenographics.ellipsoid import Ellipsoid
from StructBio.Scenographics.solids import Solids

#===============================================================================
# DehydronControlPanel class
#===============================================================================
class DehydronControlPanel(ChainsFetcherBase): 
    def __init__(self, title = ""):
        ChainsFetcherBase.__init__(self, title = title)
        self.functionForNextFrame = self.characterizeDehydrons

    #-------------------------------------------------------------------------
    def characterizeDehydrons(self):
        runCommand("set bg_color white")
        runCommand("focus")

        self.dehydron_frame = tk.LabelFrame(self.getParent(), bd = 2, text = " Characterize dehydrons ", relief = tk.RIDGE)
        self.dehydron_frame.grid(row = 3, column = 0, columnspan = 3, padx = 4, pady = 4, sticky = tk.W + tk.E)
        
        self.buildHBondsFrame()
        self.buildDehydronsFrame()

        self.d_acceptBtn = tk.Button(self.dehydron_frame, text = " Accept ", command = self.dehydronSpecAccept_Func)
        self.d_acceptBtn.grid(row = 1, column = 1, padx = 4, pady = 4, sticky = tk.E)
        self.d_acceptBtn.focus()
        self.d_acceptBtn.bind('<Return>', self.dehydronSpecAccept_Func)
        
        
    #-------------------------------------------------------------------------
    def buildHBondsFrame(self):
        setHBonds_frame = tk.LabelFrame(self.dehydron_frame, bd = 2, text = " Set HBond parameters ", relief = tk.RIDGE)
        setHBonds_frame.grid(row = 0, column = 0, padx = 4, pady = 4, sticky = tk.W+ tk.N)

        self.distSlopScaleVar = tk.DoubleVar()
        dsLabel = tk.Label(setHBonds_frame, text = " DistSlop ")
        dsLabel.grid(row = 0, column = 0, pady = 4, sticky = tk.W)
        ds_scale = tk.Scale(setHBonds_frame, variable = self.distSlopScaleVar, orient = tk.HORIZONTAL,
                         length = 100, from_ = 0.2, to = 0.8, resolution = 0.1,
                         troughcolor = "goldenrod1")
        ds_scale.grid(row = 0, column = 1, sticky = tk.W)
        ds_scale.set(0.4)

        self.angleSlopScaleVar = tk.DoubleVar()
        asLabel = tk.Label(setHBonds_frame, text = " AngleSlop ")
        asLabel.grid(row = 1, column = 0, pady = 4, sticky = tk.W)
        as_scale = tk.Scale(setHBonds_frame, variable = self.angleSlopScaleVar, orient = tk.HORIZONTAL,
                         length = 100, from_ = 10, to = 30, resolution = 1,
                         troughcolor = "goldenrod1")
        as_scale.grid(row = 1, column = 1, sticky = tk.W)
        as_scale.set(20)        

        self.lineWidthScaleVar = tk.DoubleVar()
        lwLabel = tk.Label(setHBonds_frame, text = " Line Width ")
        lwLabel.grid(row = 2, column = 0, pady = 4, sticky = tk.W)
        lw_scale = tk.Scale(setHBonds_frame, variable = self.lineWidthScaleVar, orient = tk.HORIZONTAL,
                         length = 100, from_ = 1.0, to = 8.0, resolution = 1.,
                         troughcolor = "goldenrod1")
        lw_scale.grid(row = 2, column = 1, sticky = tk.W)
        lw_scale.set(3.0)

    #-------------------------------------------------------------------------
    def buildDehydronsFrame(self):
        specifyDehydrons_frame = tk.LabelFrame(self.dehydron_frame, bd = 2, text = " Set dehydron parameters ", relief = tk.RIDGE)
        specifyDehydrons_frame.grid(row = 0, column = 1, padx = 4, pady = 4, sticky = tk.E)

        self.sphereRadiusScaleVar = tk.DoubleVar()
        srLabel = tk.Label(specifyDehydrons_frame, text = " Sphere Radius ")
        srLabel.grid(row = 0, column = 0, pady = 4, sticky = tk.W)
        sr_scale = tk.Scale(specifyDehydrons_frame, variable = self.sphereRadiusScaleVar, orient = tk.HORIZONTAL,
                         length = 100, from_ = 5.0, to = 8.0, resolution = 0.1,
                         troughcolor = "light green")
        sr_scale.grid(row = 0, column = 1, sticky = tk.W)
        sr_scale.set(6.5)

        self.wrapThresholdScaleVar = tk.IntVar()
        wtLabel = tk.Label(specifyDehydrons_frame, text = " Wrap Threshold ")
        wtLabel.grid(row = 1, column = 0, pady = 4, sticky = tk.W)
        wt_scale = tk.Scale(specifyDehydrons_frame, variable = self.wrapThresholdScaleVar, orient = tk.HORIZONTAL,
                         length = 100, from_ = 15, to = 25, resolution = 1,
                         troughcolor = "light green")
        wt_scale.grid(row = 1, column = 1, sticky = tk.W)
        wt_scale.set(19)

        chooseCenters_frame = tk.LabelFrame(specifyDehydrons_frame, bd = 2, text = " Choose centers ", relief = tk.RIDGE)
        chooseCenters_frame.grid(row = 2, column = 0, columnspan = 2, padx = 4, pady = 4, sticky = tk.W)

        self.center_rBtnResult = tk.StringVar()
        self.center_rBtnResult.set("alphas") # This button starts off selected.
        crBtn = tk.Radiobutton(chooseCenters_frame, text = " Alpha Carbons",
                               variable = self.center_rBtnResult, value = "alphas")
        crBtn.grid(row = 0, column = 0, padx = 4, sticky = tk.W)
        crBtn = tk.Radiobutton(chooseCenters_frame, text = " Donor-Acceptor",
                               variable = self.center_rBtnResult, value = "don_acc")
        crBtn.grid(row = 0, column = 1, padx = 4, sticky = tk.W)

    #-------------------------------------------------------------------------
    def buildHBondSelectionFrame(self):
        hBondSelectionFrame = tk.LabelFrame(self.getParent(), text = " Select HBond ", bd = 2, relief = tk.RIDGE)
        hBondSelectionFrame.grid(row = 4, column = 0, padx = 4, pady = 4, sticky = tk.W)
        
        self.hBondSel_lstBx = tk.Listbox(hBondSelectionFrame, height = 5, width = 35, activestyle = tk.NONE,
                                         selectmode = tk.SINGLE, bg = "white" )
                                                # Use MULTIPLE for checkbox like operation.

        topText = tk.Label(hBondSelectionFrame, text =  " Donor -- Acceptor    (wrapper count) ",
                           font = ("Helvetica", 10, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 2, pady = 4, sticky = tk.W)
        
        hBondSel_scroll = tk.Scrollbar(hBondSelectionFrame, command = self.hBondSel_lstBx.yview)
        self.hBondSel_lstBx.configure(yscrollcommand = hBondSel_scroll.set)
        hBondSel_scroll.grid(row = 1, column = 1, padx = 5, pady = 4, sticky = tk.W +tk.N + tk.S)

        self.hBondSel_lstBx.grid(row = 1, column = 0, padx = 4, pady = 4, sticky = tk.E)
        #self.hBondSel_lstBx.bind("<Double-Button-1>", self.Apply)
        self.hBondSel_lstBx.bind("<ButtonRelease-1>", self.Apply)

        # Fill the listbox:
        ix = 0
        for (donor, acptr) in self.dehydrons.hBonds_L:
            wrapCount = "  (" + str(len(self.dehydrons.wrapper_D[(donor, acptr)])) + ")"
            listEntry = atomIdString(donor) + " -- " + atomIdString(acptr) + wrapCount
            self.hBondSel_lstBx.insert(tk.END, listEntry)
            if (donor, acptr) in self.dehydrons.dehydrons_L:
                self.hBondSel_lstBx.itemconfig(ix, bg = 'white', fg = 'green')
            else:
                self.hBondSel_lstBx.itemconfig(ix, bg = 'white', fg = 'black')
            ix += 1

        # Label and button for working with HBonds selected from display:
        hBinDisplay_frame = tk.Frame(hBondSelectionFrame, bd = 2, relief = tk.RIDGE)
        hBinDisplay_frame.grid(row = 1, column = 2, columnspan = 2, padx = 4, pady = 4, sticky = tk.N + tk.E)
        
        hBinDisplayText =  tk.Label(hBinDisplay_frame, text =  " Or:\n    Work with HBonds \n    selected in display? ",
                           font = ("Helvetica", 10, "normal"), justify = tk.LEFT)
        hBinDisplayText.grid(row = 0, column = 0, pady = 4, sticky = tk.N)

        hB_Yes_Btn = tk.Button(hBinDisplay_frame, text = " Yes ", command = self.hB_Yes_Btn_Func)
        hB_Yes_Btn.grid(row = 1, column = 0, padx = 4, pady = 4, sticky = tk.E)        

    #-------------------------------------------------------------------------
    # Recursive function to disable all widgets in a frame:
    def frameDisableAllWidgets(self, widget):
        try:
            widget.config(state = tk.DISABLED)
        except tk.TclError:
            pass
        for child in widget.winfo_children():
            self.frameDisableAllWidgets(child)

    #-------------------------------------------------------------------------
    # Function to display desolvation spheres:
    def displayDesolvationSpheres(self, (donor, acptr)):
        self.hiLiter.emphasize([e for e in self.dehydrons.wrapper_D[(donor, acptr)]], [donor.residue.findAtom("CA"), acptr.residue.findAtom("CA")])
        r = self.sphereRadiusScaleVar.get()

        if self.center_rBtnResult.get() == "alphas":
            donorSphere = Ellipsoid(r, r, r, array(donor.residue.findAtom("CA").coord()),
                                    [array((0., 0., 1.)), array((0., 1., 0.)), array((1., 0., 0.))],
                                    32, 32, "Donor Sphere")
            acptrSphere = Ellipsoid(r, r, r, array(acptr.residue.findAtom("CA").coord()),
                                    [array((0., 0., 1.)), array((0., 1., 0.)), array((1., 0., 0.))],
                                    32, 32, "Acceptor Sphere")
        else:
            donorSphere = Ellipsoid(r, r, r, array(donor.coord()),
                                    [array((0., 0., 1.)), array((0., 1., 0.)), array((1., 0., 0.))],
                                    32, 32, "Donor Sphere")
            acptrSphere = Ellipsoid(r, r, r, array(acptr.coord()),
                                    [array((0., 0., 1.)), array((0., 1., 0.)), array((1., 0., 0.))],
                                    32, 32, "Acceptor Sphere")
            
        donorSphere.displaySurface((.0, .0, .6), .3)
        acptrSphere.displaySurface((.6, .0, .0), .3)
        hbSpindle = Solids()
        if (donor, acptr) in self.dehydrons.dehydrons_L:
            spindleColor = (0.0, 1.0, 0.0)
        else:
            spindleColor = (0.5, 0.5, 0.5)
        hbSpindle.addSpindle(array(donor.labelCoord()), array(acptr.labelCoord()), 0.25, spindleColor)
        # Note: End coordinates for the spindle are not suitable for ribbon representation which would require labelCoord().
        hbSpindle.display()
        

        
    #=========================================================================
    # Callback functions for getChains_frame

    def dehydronSpecAccept_Func(self, e = None):
        prot = self.getProt()
        
        if self.center_rBtnResult.get() == "alphas": alpha_Centers = True
        else: alpha_Centers = False
        
        self.dehydrons = Dehydrons(prot,
                                   distSlop = self.distSlopScaleVar.get(),
                                   angleSlop = self.angleSlopScaleVar.get(),
                                   sphereRadius = self.sphereRadiusScaleVar.get(),
                                   wrapThreshold = self.wrapThresholdScaleVar.get(),
                                   lineWidth = self.lineWidthScaleVar.get(),
                                   alphaCenters = alpha_Centers)
        
        self.buildHBondSelectionFrame()

        self.frameDisableAllWidgets(self.dehydron_frame)

        self.hiLiter = HiLiter(prot, "white")
        
        # Give all underwrapped hydrogen bonds a green color:
        for (donor, acptr) in self.dehydrons.dehydrons_L:
            hBondSpec = ("#" + str(prot.id) + ":" + str(donor.residue.id.position) + "@N" +
                         "#" + str(prot.id) + ":" + str(acptr.residue.id.position) + "@O")
            runCommand("setattr p color green " + hBondSpec)

    #-------------------------------------------------------------------------
    def Apply(self, event = None):
        line = "List box selection: " + str(self.hBondSel_lstBx.curselection())
        (selDonor, selAcptr) = self.dehydrons.hBonds_L[int(self.hBondSel_lstBx.curselection()[0])]
        open_L = chimera.openModels.list()
        numOpenModels = len(open_L)
        chimera.openModels.close([open_L[i] for i in range(1,numOpenModels)])
        self.displayDesolvationSpheres((selDonor, selAcptr))

    #-------------------------------------------------------------------------
    def hB_Yes_Btn_Func(self):
        hB_L = selection.currentPseudobonds()
        if len(hB_L) == 0: return
        
        if len(self.hBondSel_lstBx.curselection()) != 0:
            self.hBondSel_lstBx.selection_clear(self.hBondSel_lstBx.curselection())

        open_L = chimera.openModels.list()
        numOpenModels = len(open_L)

        chimera.openModels.close([open_L[i] for i in range(1, numOpenModels)])

        for e in hB_L:
            donor = e.atoms[0]
            acptr = e.atoms[1]
            self.displayDesolvationSpheres((donor, acptr))
        
        
#===============================================================================
# Mainline for testing:
#===============================================================================

chainsFetcher = DehydronControlPanel(title = "Dehydron Control Panel")


