import chimera, numpy
from numpy import array, arccos, pi

def atomDescriptorToAtomObjectMap(p):
    map_D = {}
    for a in p.atoms:
        map_D[':' + str(a.residue.id.position) + '.' + a.residue.id.chainId + '@' + a.name] = a
    return map_D

def bondAngle(atomA, atomB, atomC):
    baVec = atomA.coord() - atomB.coord()
    bcVec = atomC.coord() - atomB.coord()
    baVec.normalize(); bcVec.normalize()
    angleRadians = arccos(baVec * bcVec)
    return angleRadians * 180.0/pi


#==============================================================================
# Mainline:
#==============================================================================

prot = chimera.openModels.open("1BRF", type="PDB")[0]

map_D = atomDescriptorToAtomObjectMap(prot)

# SG Atom lables:
aLables_T = (":5.A@SG", ":8.A@SG", ":38.A@SG", ":41.A@SG")

print "Bond angles: "
for i in range(len(aLables_T)):
    for j in range(i+1, len(aLables_T)):
        # Use either of the next two lines:
        #angle = bondAngle(map_D[aLables_T[i]], map_D[":54.A@FE"], map_D[aLables_T[j]])
        angle = chimera.angle(map_D[aLables_T[i]].coord(), map_D[":54.A@FE"].coord(), map_D[aLables_T[j]].coord())
        print "(CYS " + aLables_T[i] + ") - (FE 54.A FE) - (CYS " + aLables_T[j] + ") \t%6.3f" % angle  
