import chimera, numpy
from numpy import array, arccos, pi

def bondAngle(atomA, atomB, atomC):
    baVec = atomA.coord() - atomB.coord()
    bcVec = atomC.coord() - atomB.coord()
    baVec.normalize(); bcVec.normalize()
    angleRadians = arccos(baVec * bcVec)
    return angleRadians * 180.0/pi
    
#==============================================================================
# Use a protein with a good selection of beta strands and helices (Eg. 1AON)
pdbIDchars = raw_input("Type in 4 character PDB ID or PDB file name: \n")
prot = chimera.openModels.open(pdbIDchars, type="PDB")[0]

# Data lists (atom names designate the center atom for the bond angle):
# H: helix, S: strand, L: loop.
caH = []; caS = []; caL = []
c_H = []; c_S = []; c_L = []
n_H = []; n_S = []; n_L = []

for r in prot.residues:
    rNxt = prot.residueAfter(r)
    if rNxt:
        if r.findAtom("N") and r.findAtom("CA") and r.findAtom("C") and rNxt.findAtom("N") and rNxt.findAtom("CA"):
            caAngle = bondAngle(r.findAtom("N"), r.findAtom("CA"), r.findAtom("C"))
            c_Angle = bondAngle(r.findAtom("CA"), r.findAtom("C"), rNxt.findAtom("N"))
            n_Angle = bondAngle(r.findAtom("C"), rNxt.findAtom("N"), rNxt.findAtom("CA"))
        
            if r.isHelix:
                caH.append(caAngle)
                c_H.append(c_Angle)
            if r.isStrand:
                caS.append(caAngle)
                c_S.append(c_Angle)
            if not r.isHelix and not r.isStrand:
                caL.append(caAngle)
                c_L.append(c_Angle)
            if rNxt.isHelix:  n_H.append(n_Angle)
            if rNxt.isStrand: n_S.append(n_Angle)
            if not rNxt.isHelix and not rNxt.isStrand: n_L.append(n_Angle)

print "N-CA-C angles:"
print "Secondary      Mean      Standard    Max        Min          Sample"
print "Structure:               Deviation                           Size"
print "Helix:   " + 4*"%11.2f" % (array(caH).mean(), array(caH).std(), array(caH).max(), array(caH).min()), 7*" ", len(caH)
print "Strand:  " + 4*"%11.2f" % (array(caS).mean(), array(caS).std(), array(caS).max(), array(caS).min()), 7*" ", len(caS)
print "Loop:    " + 4*"%11.2f" % (array(caL).mean(), array(caL).std(), array(caL).max(), array(caL).min()), 7*" ", len(caL)
print "\n"
            
print "CA-C-N angles:"
print "Secondary      Mean      Standard    Max        Min          Sample"
print "Structure:               Deviation                           Size"
print "Helix:   " + 4*"%11.2f" % (array(c_H).mean(), array(c_H).std(), array(c_H).max(), array(c_H).min()), 7*" ", len(c_H)
print "Strand:  " + 4*"%11.2f" % (array(c_S).mean(), array(c_S).std(), array(c_S).max(), array(c_S).min()), 7*" ", len(c_S)
print "Loop:    " + 4*"%11.2f" % (array(c_L).mean(), array(c_L).std(), array(c_L).max(), array(c_L).min()), 7*" ", len(c_L)
print "\n"
            
print "C-N-CA angles:"
print "Secondary      Mean      Standard    Max        Min          Sample"
print "Structure:               Deviation                           Size"
print "Helix:   " + 4*"%11.2f" % (array(n_H).mean(), array(n_H).std(), array(n_H).max(), array(n_H).min()), 7*" ", len(n_H)
print "Strand:  " + 4*"%11.2f" % (array(n_S).mean(), array(n_S).std(), array(n_S).max(), array(n_S).min()), 7*" ", len(n_S)
print "Loop:    " + 4*"%11.2f" % (array(n_L).mean(), array(n_L).std(), array(n_L).max(), array(n_L).min()), 7*" ", len(n_L)
print "\n" 
