import chimera, Image, tkFileDialog
import matplotlib.pyplot as plt

def ramaPlot(phi_L, psi_L, saveFilePath = ""):
    plt.ion()
    fig = plt.figure(figsize = (8,8))

    ax = fig.add_subplot(111, aspect = 'equal')
    ax.set_title("Ramachandran Plot  " + saveFilePath, fontsize = 14)
    ax.set_xlabel("PHI (degrees)", fontsize = 12)
    ax.set_ylabel("PSI (degrees)", fontsize = 12)
    ax.set_xlim([-180, 180])
    ax.set_ylim([-180, 180])
    ax.set_xticks([-180, -135, -90, -45, 0, 45, 90, 135, 180])
    ax.set_yticks([-180, -135, -90, -45, 0, 45, 90, 135, 180])
    img = Image.open('croppedRamachandranBackGround.png')
    im = plt.imshow(img, extent = (-180, 180, 180, -180))
    ax.grid(True, linestyle = '-', color = '0.75')
    ax.hlines(0, -180, 180, colors='k', linestyles='dashed')
    ax.vlines(0, -180, 180, colors='k', linestyles='dashed')

    # Initialization:
    # List indicies: 0 for Helix, 1 for Strand, and 2 for Coil.
    colors_T = ('k', 'g', 'b') # Marker colors for Helix, Strand, and Coil.
    shapes_T = ('o', 's', 'd') # Marker shapes for Helix, Strand, and Coil.

    # Drop in the markers:
    for i in range(3):
        ax.scatter(phi_L[i], psi_L[i], s = 20, color = colors_T[i], marker = shapes_T[i])

    # Save file if the saveFilePath parameter is not the empty string.
    if saveFilePath != "":
        fig.savefig(saveFilePath + "_RamachandranPlot", dpi = fig.dpi)
        
    raw_input("Hit Enter to dismiss the plot.")
    plt.close()


#==============================================================================
# Mainline:
#==============================================================================

phi_L = [[], [], []]    # Phi lists for Helix, Strand, and Coil
psi_L = [[], [], []]    # Psi lists for Helix, Strand, and Coil 
totalSampleCount = 0; hSampleCount = 0; sSampleCount = 0; cSampleCount = 0;

# Launch dialog to get path of PDB ID list:
filePathName = tkFileDialog.askopenfilename()
print "Path name for PDB file list is: ", filePathName

# Read file containing list of PDB identifiers:
listFileHandle = file(filePathName)
listFileHandle.readline()    # Skip first line with statement of purpose.
targetRes = listFileHandle.readline()[0:3] # Get residue type

print "Doing Ramachandran plot for residue: ", targetRes
print "\n"

# Now go through list of PDB IDs:
pdbID = listFileHandle.readline()
while pdbID:
    print "Processing: ", pdbID[0:4]
    prot_L = chimera.openModels.open(pdbID[0:4], type = "PDB")

    for r in prot_L[0].residues:
        if r.type != targetRes: continue
        if not(r.phi and r.psi): continue # Skip if residue has no phi, psi attributes.
        totalSampleCount += 1
        if r.isHelix:
            hSampleCount += 1
            phi_L[0].append(r.phi)
            psi_L[0].append(r.psi)
        if r.isStrand:
            sSampleCount += 1
            phi_L[1].append(r.phi)
            psi_L[1].append(r.psi)    
        if not (r.isHelix or r.isStrand):
            cSampleCount += 1
            phi_L[2].append(r.phi)
            psi_L[2].append(r.psi)  
    
    pdbID = listFileHandle.readline()   # Go for next pdbID in the list.
    chimera.openModels.close(prot_L)
    
listFileHandle.close()  # Close the list file.

print "Number of ", targetRes, " residues found: ", totalSampleCount
print "Number of residues within helices: %5d" % hSampleCount
print "Number of residues within strands: %5d" % sSampleCount
print "Number of residues within coils:   %5d" % cSampleCount

ramaPlot(phi_L, psi_L, targetRes)

