import chimera, numpy
from numpy import array, linalg

from StructBio.LeastSquares.planes import LeastSquaresPlane
from StructBio.Scenographics.surfaces import Surfaces

# =======================   Part (a)  ==========================
#
# We will use the term "ligand" atoms when referring to
# OXY, CMO, or CYN atoms. NOTE: the least squares plane is drawn
# only for the first chain.

pdbIDchars = raw_input("Type in 4 character PDB ID or PDB file name: \n")
prot = chimera.openModels.open(pdbIDchars, type="PDB")[0]

chainID = prot.sequences(asDict = True).keys()[0]
targetRes = "HEM"
targetNames_L = ["CHA", "C1A", "C4A", "CHB", "C1B", "C4B",
                 "CHC", "C1C", "C4C", "CHD", "C1D", "C4D"]
targetAtoms_L = []

for r in prot.residues:
    if r.type == targetRes and r.id.chainId == chainID:
        for a in r.atoms:
            if a.name in targetNames_L:
                targetAtoms_L.append(a)        

lsPlane = LeastSquaresPlane(targetAtoms_L)

# Compute projections of target list atoms on the plane.
atomCoordsP_L = []
for atm in targetAtoms_L:
    atomCoordsP_L.append(lsPlane.projectionOnPlane(atm))

planeColor = (0.2, 0.2, 0.8)
poly = Surfaces()
poly.addPolygon(atomCoordsP_L, planeColor)
poly.display()

from chimera import runCommand
runCommand("set bg_color white")


# =======================   Part (b)  ==========================

# Get ligand atoms.
lig_atoms = []
for r in prot.residues:
    if r.type in ("OXY", "CMO", "CYN"):
        ligType = r.type
        for a in r.atoms:
            lig_atoms.append(a)

angle = lsPlane.angleToPlane(lig_atoms[0], lig_atoms[1])

print "Required angle for ligand " + ligType + " within " + pdbIDchars[0:4] + " = %7.3f" % angle


