import chimera, Tkinter as tk, matplotlib.pyplot as plt, Image
from chimera.baseDialog import ModelessDialog
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.figure import Figure
from matplotlib.pyplot import imshow

class RamaPlotDialog(ModelessDialog):
    buttons = ("Save", "Close")
    help = False    # Leave this out to get a Help button.
    title = "Ramachandran Plot"
    
    #-----------------------------------------------------------------------
    def fillInUI(self, parent):
        self.prot = None
        # Text to be displayed in the dialog window:
        topText = tk.Label(parent, text =  " Type in a PDB identifier:",
                           font = ("Helvetica", 11, "normal"))
        topText.grid(row = 0, column = 0, columnspan = 2, pady = 4, sticky = tk.W)

        self.entryVar = tk.StringVar()
        self.entry = tk.Entry(parent, textvariable = self.entryVar,
                              width = 8, bg = "white")
        self.entry.grid(row = 1, column = 0)
        self.entry.focus_set()
        
        self.btnFetch = tk.Button(parent, text = " Fetch ", default = "active",
                                  command = lambda: self.btnFunc(parent))
        self.btnFetch.grid(row = 1, column = 1, padx = 4, pady = 4)

        self.entry.bind('<Return>', lambda event: self.btnFunc(parent))
        
    #-----------------------------------------------------------------------
    def Save(self):
        # Save file if the prot is not None.
        if self.prot != None:
            self.fig.savefig(self.prot.name + "_RamachandranPlot", dpi = self.fig.dpi)

    #-----------------------------------------------------------------------
    def btnFunc(self, parent):
        if self.prot: chimera.openModels.close([self.prot])
        self.prot = chimera.openModels.open(self.entryVar.get(), type="PDB")[0]
        res_L = self.prot.residues
        
        self.fig = Figure(figsize = (7,6), dpi = 100)

        ax = self.fig.add_subplot(111, aspect = 'equal')
        ax.set_title("Ramachandran Plot for " + self.prot.name, fontsize = 14)
        ax.set_xlabel("PHI (degrees)", fontsize = 12)
        ax.set_ylabel("PSI (degrees)", fontsize = 12)
        ax.set_xlim([-180, 180])
        ax.set_ylim([-180, 180])
        ax.set_xticks([-180, -135, -90, -45, 0, 45, 90, 135, 180])
        ax.set_yticks([-180, -135, -90, -45, 0, 45, 90, 135, 180])
        img = Image.open('croppedRamachandranBackGround.png')
        ax.imshow(img, extent = (-180, 180, 180, -180))
        ax.grid(True, linestyle = '-', color = '0.75')
        ax.hlines(0, -180, 180, colors='k', linestyles='dashed')
        ax.vlines(0, -180, 180, colors='k', linestyles='dashed')

        # Initialization:
        # List indicies: 0 for Helix, 1 for Strand, and 2 for Coil.
        colors_T = ('k', 'g', 'b') # Marker colors for Helix, Strand, and Coil.
        shapes_T = ('o', 's', 'd') # Marker shapes for Helix, Strand, and Coil.
        phi_L = [[], [], []]    # Phi lists for Helix, Strand, and Coil
        psi_L = [[], [], []]    # Psi lists for Helix, Strand, and Coil 

            
        # Generate the phi, psi coordinate lists:
        for r in res_L:
            if not(r.phi and r.psi): continue # Skip if residue has no phi, psi attributes.
            if r.isHelix:
                phi_L[0].append(r.phi)
                psi_L[0].append(r.psi)
            if r.isStrand:
                phi_L[1].append(r.phi)
                psi_L[1].append(r.psi)    
            if not (r.isHelix or r.isStrand):
                phi_L[2].append(r.phi)
                psi_L[2].append(r.psi)

        # Drop in the markers:
        for i in range(3):
            ax.scatter(phi_L[i], psi_L[i], s = 20, color = colors_T[i], marker = shapes_T[i])

        #-------------------------------------------------------------------------
        # Set up a canvas and put the Figure into it.
        canvasFrame = tk.Frame(parent, bd = 2, relief = tk.RIDGE)
        canvasFrame.grid(row = 2, column = 0, columnspan = 8, padx = 4, pady = 4, sticky = tk.E)

        canvas = FigureCanvasTkAgg(self.fig, master = canvasFrame)
        canvas.get_tk_widget().grid(row = 0, column = 0)

        # Set up legend text.
        legendText = tk.Label(canvasFrame, text = "Helix: black circle.    Strand: green square.    Coil: blue diamond.")
        legendText.grid(row = 1, column = 0, columnspan = 8, padx = 4, sticky = tk.W)
        
#==============================================================================
# Mainline:

RamaPlotDialog()
