import chimera, numpy
from chimera import runCommand
from numpy import linalg
from StructBio.StructureOverlap.overlapper import Overlapper

def getAtom(prot, resPosition, atomName):
    for r in prot.residues:
        if r.id.position == resPosition:
            return r.findAtom(atomName)

#==============================================================================
# Mainline:
#==============================================================================
protP = chimera.openModels.open("1HHP", type = "PDB")[0]
protQ = chimera.openModels.open("1EBY", type = "PDB")[0]

runCommand("set bg_color white; sel #0-1:.A | :BEB; sel invert; delete sel; ~sel")
runCommand("modelcolour pink #0; modelcolour turquoise #1")
overlap_L = []

atomPairPositions_L = [62, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73,
                       87, 88, 89, 90, 91, 92, 93]

for pos in atomPairPositions_L:
    overlap_L.append([getAtom(protP, pos, "CA"), getAtom(protQ, pos, "CA")])
                                  
overlapper = Overlapper(protP)
overlapper.moveToOverlap(overlap_L)

runCommand("window")
 
print "RMSD for overlap: %6.4f" % overlapper.rmsdForOverlap()


gly49Shift = linalg.norm(getAtom(protP, 49, "CA").coord() - getAtom(protQ, 49, "CA").coord())
print "Shift of CA atom in GLY 49.A: %6.4f" % gly49Shift


