import chimera, Rotamers
from chimera import runCommand, selection
from Rotamers import getRotamers, useRotamer, NoResidueRotamersError

# -----------------------------------------------------------------------------
def userSelectsResidue(focus = False):
    print "Select an atom or a standard residue in the display."
    raw_input("Then bring the cursor back to this line and hit Enter.")
    atm_L = selection.currentAtoms()
    if len(atm_L) == 0: return None
    res = atm_L[0].residue
    if focus:
        runCommand("sel :" + str(res.id.position) + "." +
                   res.id.chainId + " za<8.0; focus sel; ~sel")
    return res

# -----------------------------------------------------------------------------
def printChiAngles(r):
    if r.chi1: print "chi1: %8.3f" % r.chi1
    if r.chi2: print "chi2: %8.3f" % r.chi2
    if r.chi3: print "chi3: %8.3f" % r.chi3
    if r.chi4: print "chi4: %8.3f" % r.chi4
    return

#==============================================================================
# Mainline:
#==============================================================================

targetPath = raw_input("Type 4 character PDB ID: \n")
prot = chimera.openModels.open(targetPath, type="PDB")[0]

runCommand("set bg_color white; ~ribbon; repr stick; show")

selR = userSelectsResidue(focus = True)
if not selR: selR = prot.residues[0] # Use first residue if user does not select.
resStr = ":" + str(selR.id.position) + "." + selR.id.chainId

runCommand("repr bs " + resStr)

print "Use the Enter key to step through the rotamers."
print "Type any character then Enter to terminate the script.\n"
print selR.type, "residue " + resStr + " selected."

print "Current CHI angles for selected residue:"
printChiAngles(selR)
print "--------------------------------------------------"
print "\n"

try:
    (flag, rots_L) = getRotamers(selR)
    numRots = len(rots_L)
    nullChar = ""
    rotIx = 0
    while nullChar == "":
        print "\n"
        resRot = rots_L[rotIx].residues[0]
        print "Using rotamer having index ", rotIx, \
              "probability ", rots_L[rotIx].rotamerProb
        print "chi angles for rotamer before useRotamer call: "
        printChiAngles(resRot)
        
        useRotamer(selR, [rots_L[rotIx]])
        print "chi angles for rotamer after useRotamer call: "
        printChiAngles(selR)

        nullChar = raw_input("------------ Key Enter to continue ------------")
        rotIx += 1
        rotIx = rotIx%numRots
        if rotIx == 0: print "\n"
    
except NoResidueRotamersError:
    print selR.id.position, selR.type, "No rotamers"
    


        
